/*
xbinclk.c Version 0.6.0. Binary clock in X Windows.
Copyright (C) 2001-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* to define the escape key */
#define XK_MISCELLANY 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <asm/errno.h>
#include <X11/Xlib.h>
#include <X11/keysymdef.h>
#include <assert.h>
#include <unistd.h>

#define FONT "8x16"
#define NIL (0)
#define SCRW 400
#define SCRH 320
#define CIRCLE 23040
#define SZ 10

int whiteColor,blackColor;
int runflg,alfa;
int prevyr,prevmon,prevday,prevjul;
int prevdow,prevhr,prevmin,prevsec;
int prevdst;
unsigned long blue;
Display *dpy;
Window w;
GC gc;
Font fontid;
Screen *dfltscr;
Colormap cmap;
XColor scrdef,exctdef;

/* X Windows code is based on: */
/* http://tronche.lri.fr:8000/gui/x/xlib-tutorial/2nd-program-anatomy.html */

void getkey()
   {
   int i,rslt,msk;
   int symbol;
   int XCheckMaskEvent();
   XEvent e;
   XKeyEvent *k;
   char spc[32];

   msk = KeyPressMask|ExposureMask;

   XSelectInput(dpy, w, msk);

   while (XCheckMaskEvent(dpy, msk, &e))
      {
      if (e.type == KeyPress)
         {
         k = (XKeyEvent *) &e;
         symbol = XLookupKeysym(k,0);
         if (symbol == XK_Escape)
            {
            runflg = 0;
            } /* if quit */
         else if (symbol == 'a')
            {
            alfa = (alfa ^ 1) & 1;
            if (!alfa)
               {
               strcpy(spc,"                    ");
               XDrawImageString(dpy,w,gc,30,220,spc,20);
               XDrawImageString(dpy,w,gc,190,220,spc,20);
               XDrawImageString(dpy,w,gc,30,240,spc,20);
               XDrawImageString(dpy,w,gc,190,240,spc,20);
               XDrawImageString(dpy,w,gc,30,260,spc,20);
               XDrawImageString(dpy,w,gc,190,260,spc,20);
               XDrawImageString(dpy,w,gc,30,280,spc,20);
               XDrawImageString(dpy,w,gc,190,280,spc,20);
               } /* if alfa toggled off */
            prevyr = prevjul = prevdst = -99999;
            } /* alfa toggle */
         } /* if keypress event */
      else if (e.type == Expose)
         {
         prevyr = prevmon = prevday = prevjul
            = prevdow = prevhr = prevmin = prevsec
            = prevdst =  -99999;
         } /* if expose event */
      } /* if event received */
   } /* getkey */

void initx()
   {
   int i,rslt;
   char title[64];

   dpy = XOpenDisplay(NIL);

   if (dpy == NULL)
      {
      fprintf(stderr,"X Windows failure\n");
      exit(1);
      } /* if X Windows is not active */

   assert(dpy);

   whiteColor = WhitePixel(dpy, DefaultScreen(dpy));
   blackColor = BlackPixel(dpy, DefaultScreen(dpy));

   w = XCreateSimpleWindow(dpy,
      DefaultRootWindow(dpy),
      0, 0, 
      SCRW, SCRH,
      0, whiteColor,
      whiteColor);

   XSelectInput(dpy, w, StructureNotifyMask);

   XMapWindow(dpy, w);

   gc = XCreateGC(dpy, w, 0, NIL);

   fontid = (Font) XLoadFont(dpy,FONT);

   XSetFont(dpy,gc,fontid);

   XSetForeground(dpy, gc, blackColor);

   dfltscr = XDefaultScreenOfDisplay(dpy);
   if (dfltscr == NULL)
      {
      fprintf(stderr,"XDefaultScreenOfDisplay failed\n");
      perror("XDefaultScreenOfDisplay failed");
      exit(1);
      } /* if error */

   cmap = XDefaultColormapOfScreen(dfltscr);

   rslt = XAllocNamedColor(dpy,cmap,"blue",
      &scrdef,&exctdef);
   if (rslt < 0)
      {
      fprintf(stderr,"XAllocNamedColor failed\n");
      perror("XAllocNamedColor failed");
      exit(1);
      } /* if error */
   blue = scrdef.pixel;

   XSetWindowBorderWidth(dpy, w, 40);

   sprintf(title,"xbinclk");
   XStoreName(dpy,w,title);

   XMoveWindow(dpy,w,0,0);

   while(1)
      {
      XEvent e;
      XNextEvent(dpy, &e);
      if (e.type == MapNotify) break;
      } /* wait for window initialization */

   } /* initx */

int iflpyr(int year)
   {
   if (year < 1583) return(0);
   if (!(year % 400)) return(1);
   else if (!(year % 100)) return(0);
   else if (!(year % 4)) return(1);
   else return(0);
   } /* iflpyr */

void putyear(int year)
   {
   int iflpyr();
   XSetBackground(dpy, gc, whiteColor);
   if (year & 0x0800)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,20,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,20,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,20,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,20,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0400)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,40,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,40,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,40,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,40,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0200)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,60,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,60,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,60,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,60,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0100)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,80,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,80,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,80,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,80,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0080)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,100,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,100,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,100,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,100,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0040)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,120,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,120,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,120,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,120,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0020)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,140,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,140,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,140,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,140,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0010)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,160,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,160,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,160,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,160,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,20,SZ,SZ,0,CIRCLE);
      }
   if (year & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,20,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,20,SZ,SZ,0,CIRCLE);
      }
   if (iflpyr(year))
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,300,20,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,300,20,SZ,SZ,0,CIRCLE);
      } /* if leap year */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,300,20,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,300,20,SZ,SZ,0,CIRCLE);
      } /* else not leap year */
   XSetForeground(dpy, gc, blackColor);
   } /* putyear */

void putmon(int mon)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (mon & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,40,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,40,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,40,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,40,SZ,SZ,0,CIRCLE);
      }
   if (mon & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,40,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,40,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,40,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,40,SZ,SZ,0,CIRCLE);
      }
   if (mon & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,40,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,40,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,40,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,40,SZ,SZ,0,CIRCLE);
      }
   if (mon & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,40,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,40,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,40,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,40,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putmon */

void putday(int day)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (day & 0x0010)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,160,60,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,160,60,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,160,60,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,160,60,SZ,SZ,0,CIRCLE);
      }
   if (day & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,60,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,60,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,60,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,60,SZ,SZ,0,CIRCLE);
      }
   if (day & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,60,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,60,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,60,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,60,SZ,SZ,0,CIRCLE);
      }
   if (day & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,60,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,60,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,60,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,60,SZ,SZ,0,CIRCLE);
      }
   if (day & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,60,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,60,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,60,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,60,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putday */

void putjul(int julday)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (julday & 0x0100)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,80,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,80,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,80,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,80,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0080)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,100,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,100,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,100,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,100,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0040)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,120,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,120,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,120,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,120,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0020)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,140,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,140,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,140,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,140,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0010)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,160,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,160,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,160,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,160,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,80,SZ,SZ,0,CIRCLE);
      }
   if (julday & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,80,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putjul */

void putdow(int wday)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (wday & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,300,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,300,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,300,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,300,80,SZ,SZ,0,CIRCLE);
      }
   if (wday & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,320,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,320,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,320,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,320,80,SZ,SZ,0,CIRCLE);
      }
   if (wday & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,340,80,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,340,80,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,340,80,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,340,80,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putdow */

void puthour(int hour)
   {
   int i;
   XSetBackground(dpy, gc, whiteColor);
   i = hour % 12;
   if (i == 0) i = 12;
   if (i & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,120,SZ,SZ,0,CIRCLE);
      }
   if (i & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,120,SZ,SZ,0,CIRCLE);
      }
   if (i & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,120,SZ,SZ,0,CIRCLE);
      }
   if (i & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,120,SZ,SZ,0,CIRCLE);
      }
   if (hour > 11)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,300,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,300,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,300,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,300,120,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* puthour */

void putdst(int dst)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (dst)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,340,120,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,340,120,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,340,120,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,340,120,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putdst */

void putmin(int min)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (min & 0x0020)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,140,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,140,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,140,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,140,140,SZ,SZ,0,CIRCLE);
      }
   if (min & 0x0010)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,160,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,160,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,160,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,160,140,SZ,SZ,0,CIRCLE);
      }
   if (min & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,140,SZ,SZ,0,CIRCLE);
      }
   if (min & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,140,SZ,SZ,0,CIRCLE);
      }
   if (min & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,140,SZ,SZ,0,CIRCLE);
      }
   if (min & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,140,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,140,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,140,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,140,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putmin */

void putsec(int sec)
   {
   XSetBackground(dpy, gc, whiteColor);
   if (sec & 0x0020)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,140,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,140,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,140,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,140,160,SZ,SZ,0,CIRCLE);
      }
   if (sec & 0x0010)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,160,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,160,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,160,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,160,160,SZ,SZ,0,CIRCLE);
      }
   if (sec & 0x0008)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,180,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,180,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,180,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,180,160,SZ,SZ,0,CIRCLE);
      }
   if (sec & 0x0004)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,200,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,200,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,200,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,200,160,SZ,SZ,0,CIRCLE);
      }
   if (sec & 0x0002)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,220,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,220,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,220,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,220,160,SZ,SZ,0,CIRCLE);
      }
   if (sec & 0x0001)
      {
      XSetForeground(dpy, gc, blue);
      XDrawArc(dpy,w,gc,240,160,SZ,SZ,0,CIRCLE);
      XFillArc(dpy,w,gc,240,160,SZ,SZ,0,CIRCLE);
      } /* if bit */
   else
      {
      XSetForeground(dpy, gc, whiteColor);
      XFillArc(dpy,w,gc,240,160,SZ,SZ,0,CIRCLE);
      XSetForeground(dpy, gc, blackColor);
      XDrawArc(dpy,w,gc,240,160,SZ,SZ,0,CIRCLE);
      }
   XSetForeground(dpy, gc, blackColor);
   } /* putsec */

void putascii(int year,int jul,int dst,char *str)
   {
   int iflpyr();
   int len;
   char julian[128];
   char alfadate[64];
   char spc[32];
   strcpy(alfadate,str);
   len = strlen(alfadate);
   alfadate[len-1] = '\0';
   strcpy(spc,"                    ");
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, blackColor);
   XDrawImageString(dpy,w,gc,70,220,alfadate,len);
   if (jul != prevjul)
      {
      sprintf(julian,"Julian Day %d", jul);
      XDrawImageString(dpy,w,gc,30,240,spc,20);
      XDrawImageString(dpy,w,gc,190,240,spc,20);
      XDrawImageString(dpy,w,gc,70,240,julian,strlen(julian));
      } /* if julian day changed */
   if (dst != prevdst)
      {
      XDrawImageString(dpy,w,gc,30,260,spc,20);
      XDrawImageString(dpy,w,gc,190,260,spc,20);
      if (dst)
         {
         XDrawImageString(dpy,w,gc,70,260,"Daylight Saving",15);
         } /* if daylight saving */
      else
         {
         XDrawImageString(dpy,w,gc,70,260,"Standard Time",13);
         } /* if standard time */
      } /* if daylight saving changed */
   if (year != prevyr)
      {
      XDrawImageString(dpy,w,gc,30,280,spc,20);
      XDrawImageString(dpy,w,gc,190,280,spc,20);
      if (iflpyr(year))
         {
         XDrawImageString(dpy,w,gc,70,280,"Leap Year",9);
         } /* if leap year */
      else
         {
         XDrawImageString(dpy,w,gc,70,280,"Non-leap Year",13);
         } /* else not leap year */
      } /* if year changed */
   } /* putascii */

void runclk()
   {
   int curryr;
   time_t now;
   clock_t clk;
   struct tms t;
   struct tm *lcltm;
   char *str;
   /* get clock ticks since boot */
   clk = times(&t);
   runflg = 1;
   alfa = 1;            /* default to alpha display */
   while (runflg)
      {
      /* get date & time */
      time(&now);
      lcltm = (struct tm *) localtime(&now);
      curryr = lcltm->tm_year+1900;
      /* putascii must execute before */
      /* putyear, putjul, and putdst logic below */
      if (alfa)
         {
         str = (char *) asctime(lcltm);
         putascii(curryr,
            lcltm->tm_yday+1,
            lcltm->tm_isdst,
            str);
         } /* if alfa */
      if (curryr != prevyr)
         {
         putyear(curryr);
         prevyr = curryr;
         } /* if change */
      if (lcltm->tm_mon+1 != prevmon)
         {
         putmon(lcltm->tm_mon+1);
         prevmon = lcltm->tm_mon+1;
         } /* if change */
      if (lcltm->tm_mday != prevday)
         {
         putday(lcltm->tm_mday);
         prevday = lcltm->tm_mday;
         } /* if change */
      if (lcltm->tm_yday+1 != prevjul)
         {
         putjul(lcltm->tm_yday+1);
         prevjul = lcltm->tm_yday+1;
         } /* if change */
      if (lcltm->tm_wday+1 != prevdow)
         {
         putdow(lcltm->tm_wday+1);
         prevdow = lcltm->tm_wday+1;
         } /* if change */
      if (lcltm->tm_hour != prevhr)
         {
         puthour(lcltm->tm_hour);
         prevhr = lcltm->tm_hour;
         } /* if change */
      if (lcltm->tm_isdst != prevdst)
         {
         putdst(lcltm->tm_isdst);
         prevdst = lcltm->tm_isdst;
         } /* if change */
      if (lcltm->tm_min != prevmin)
         {
         putmin(lcltm->tm_min);
         prevmin = lcltm->tm_min;
         } /* if change */
      if (lcltm->tm_sec != prevsec)
         {
         putsec(lcltm->tm_sec);
         prevsec = lcltm->tm_sec;
         } /* if change */
      XFlush(dpy);
      getkey();
      if (runclk) sleep(1);
      }
   } /* runclk */

int main()
   {
   prevyr = prevmon = prevday = prevjul
      = prevdow = prevhr = prevmin = prevsec
      = prevdst = -99999;
   initx();
   runclk();
   return(0);
   } /* main */
